"""_summary_.

https://core.telegram.org/bots/features#creating-a-new-bot
- send a message to the bot @BotFather
- follow the instructions to create a new bot

Get the chat ID:
    Group:
    - Send the message `/my_id @MyBotNameBot` to the group
    DM
    - Send any message to the bot

`Telegram(token=your_token).get_chats()`

Example:
---------
```python
import os

from dotenv import load_dotenv
from envoye import Telegram

load_dotenv()

TOKEN = os.getenv("TELEGRAM_API_TOKEN")
CHAT_ID = os.getenv("TELEGRAM_CHAT_ID")
provider = Telegram(token=TOKEN, chat_id=CHAT_ID)
provider.get_chats()
```
"""

from dataclasses import dataclass
from typing import Optional

import requests
from rich import print
from rich.table import Table


@dataclass
class Telegram:
    """Send Telegram notifications."""

    token: Optional[str] = None
    chat_id: Optional[str] = None

    def __post_init__(self) -> None:
        if self.token is None:
            raise ValueError("Telegram token is required")

    def get_chats(self) -> None:
        """Return the chat ID for the bot."""
        url = f"https://api.telegram.org/bot{self.token}/getUpdates"
        data = requests.get(url).json()
        if not data["ok"]:
            print(data)
            raise ValueError("Error getting bot's chats.")

        if len(data["result"]) == 0:
            raise ValueError("Bot has no chats.")

        table = Table(title="Telegram Bot's Chats")
        table.add_column("Chat ID")
        table.add_column("Type")
        table.add_column("User's Name")
        table.add_column("Group Title")
        for c in data["result"]:
            chat = c["message"]["chat"]
            row = []
            row.extend([str(chat["id"]), chat["type"]])

            if chat["type"] == "group":
                row.extend(["", chat["title"]])
            else:
                row.append(f"{chat['first_name']} {chat.get('last_name', '')}")

            table.add_row(*row)

        print(table)

    def send(self, message: str) -> None:
        """Send the message."""
        if self.chat_id is None:
            raise ValueError("Chat ID is required")

        url = f"https://api.telegram.org/bot{self.token}/sendMessage"
        url += f"?chat_id={self.chat_id}&text={message}"

        requests.get(url)
